package Nitesi::DBI;

use 5.006;
use strict;
use warnings;

=head1 NAME

Nitesi::DBI - DBI bundle for Nitesi shop machine

=head1 VERSION

Version 0.0060

=cut

our $VERSION = '0.0060';

=head1 DESCRIPTION

This bundle contains:

=over 4

=item 

L<Nitesi::Query::DBI>

=item

L<Nitesi::Account::Provider::DBI>

=back

=head1 AUTHOR

Stefan Hornburg (Racke), C<< <racke at linuxia.de> >>

=head1 BUGS

Please report any bugs or feature requests to C<bug-nitesi-dbi at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=Nitesi-DBI>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.

=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc Nitesi::DBI


You can also look for information at:

=over 4

=item * RT: CPAN's request tracker (report bugs here)

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Nitesi-DBI>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/Nitesi-DBI>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/Nitesi-DBI>

=item * Search CPAN

L<http://search.cpan.org/dist/Nitesi-DBI/>

=back

=head1 LICENSE AND COPYRIGHT

Copyright 2011 Stefan Hornburg (Racke).

This program is free software; you can redistribute it and/or modify it
under the terms of either: the GNU General Public License as published
by the Free Software Foundation; or the Artistic License.

See http://dev.perl.org/licenses/ for more information.


=cut

1; # End of Nitesi::DBI
